package hep.wired.corbaheprep;

import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.FieldPosition;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * @author Mark Donszelmann
 * @version $Id: BaBarODFTime.java 8598 2006-08-14 20:38:02Z duns $
 */
public class BaBarODFTime {

    // Number of sysclks in a SubFiducial period
    protected static final int SUBFIDPERIOD = 873;

    // (27*365+7)*24*60*60 = 852,076,800; 7 leap years in the period
    private static final long SECONDS_1970_TO_1997 = 852076800L;

    private BaBarODFTime() {
    }

    public static String convertToString(String odfTime) {
        return convertToString(odfTime, TimeZone.getTimeZone("PST"));
    }

    public static String convertToString(String odfTime, TimeZone zone) {
        String[] hilo = odfTime.split("/", 2);
        String hex = (hilo.length == 2) ? hilo[0] + hilo[1] : hilo[0];
        return convertToString(Long.parseLong(hex, 16), zone);
    }

    public static String convertToString(long odfTime, TimeZone zone) {
        // FIXME check for overflow...
        // odfTime / 59.5 MHz
        long clks = (odfTime + odfTime) / 119;

        // seconds and microseconds
        long subfiducial = odfTime % 873;
        long nseconds = (clks % 1000000) * 1000;
        long seconds = (clks / 1000000) + SECONDS_1970_TO_1997;
        long mseconds = seconds * 1000;

        Date date = new Date(mseconds);
        DateFormat f1 = new SimpleDateFormat("EEE MMM dd HH:mm:ss.");
        DateFormat f2 = new SimpleDateFormat(" yyyy zzz");
        NumberFormat n = new DecimalFormat("000000000");

        f1.setTimeZone(zone);
        f2.setTimeZone(zone);

        StringBuffer s = new StringBuffer();
        FieldPosition p = new FieldPosition(0);
        s = f1.format(date, s, p);
        s = n.format(new Long(nseconds), s, p);
        s = f2.format(date, s, p);
        s.append(" (");
        s.append(subfiducial);
        s.append(")");
        return s.toString();
    }

    public static void main(String[] args) throws Exception {
        String odfTime = "0189e5/e884e779";
        TimeZone utc = TimeZone.getTimeZone("UTC");
        System.out.println(BaBarODFTime.convertToString(odfTime, utc));
        System.out.println(BaBarODFTime.convertToString(odfTime));
        System.out.println(BaBarODFTime.convertToString("02e20ea6f013a53"));
    }
}
